// vim: set ts=2 sw=2 et:
#pragma once
#include "protocol.h"
#include "reader.h"

#include <iostream>
#include <thread>

template <typename T>
struct Parser
{
  inline Parser(T &_instance)
    : instance(_instance), readBufferReadPos(readBuffer)
  {
  }

  template<typename Type> inline static Type *readBody(char *(&ptr), uint32_t len) {
    Type * typeptr = reinterpret_cast<Type*>(ptr);
    ptr += len;
    return typeptr;
  }

  inline void parseLoop()
  {
    while (true)
    {
      char *_readBufferAvailable = readBufferAvailable.load();
      char *readBufferDemand = nullptr;

      while (true)
      {
        if (readBufferReadPos + sizeof(MessageHead) > _readBufferAvailable)
        {
          readBufferDemand = readBufferReadPos + sizeof(MessageHead);
          break;
        }
        MessageHead *headptr = reinterpret_cast<MessageHead *>(readBufferReadPos);
        MessageHead &head = *headptr;
        if (readBufferReadPos + sizeof(MessageHead) + head.messageLen > _readBufferAvailable)
        {
          readBufferDemand = readBufferReadPos + sizeof(MessageHead) + head.messageLen;
          break;
        }
        readBufferReadPos += sizeof(MessageHead);
        switch (head.type) {
          case MessageHead::Done: instance.processDone(); return;
          case MessageHead::Transaction: instance.processTransaction(readBody<Transaction>(readBufferReadPos,head.messageLen)); break;
          case MessageHead::ValidationQueries: instance.processValidationQueries(readBody<ValidationQueries>(readBufferReadPos,head.messageLen)); break;
          case MessageHead::Flush: 
          {
            instance.processFlush(readBody<Flush>(readBufferReadPos,head.messageLen));
            readBufferReadPos = readBuffer;
            goto endOfInnerLoop;
            break;
          }
          case MessageHead::Forget: instance.processForget(readBody<Forget>(readBufferReadPos,head.messageLen)); break;
          default: break;
        }
      }

      if (_readBufferAvailable == readBufferLimit) // buffer is full
      {
        instance.processFlush(nullptr);
        readBufferReadPos = readBuffer;
      }
      else // wait for the buffer to be filled
        readBufferWaitForBytes(readBufferDemand);

      endOfInnerLoop:
      ;
    }
  }

  T &instance;
  char *readBufferReadPos;
};
