#pragma once

#include <thread>
#include <iostream>
#include <sstream>
#include "relation.h"
#include "parser.h"
#include "folly/LifoSem.h"
#include "validator.h"
#include "validation-result.h"

#ifdef _PROFILE
#include "profiling.h"
#include <iomanip>
#endif

namespace Prevalidator
{
struct ThreadContext;

extern const int THREAD_COUNT;

struct ThreadContext
{
    ThreadContext(int threadIndex);

    void mainLoop();

    // Parser interface
    void processDone() { }
    void processTransaction(Transaction *t) { }
    void processValidationQueries(ValidationQueries *v);
    void processFlush(Flush *f);
    void processForget(Forget *f) { }

    int threadIndex;
    int validationQueryThreadIndex;
    folly::fbvector<folly::fbvector<Validator::ValidationSubtask>> *validationQueueItems;
    folly::LifoSem flushCompleteSemaphore;

#ifdef _PROFILE
    TimestampStack timeStack;
    ProfilingStats timePreprocess;
    ProfilingStats timeProduceTasks;
    ProfilingStats *timeRelation;

    std::string profilingStatsToString()
    {
        std::ostringstream oss;
        oss << "Preprocess: " << timePreprocess.toString() << "\nProduceTasks: " << timeProduceTasks.toString() << "\nRelations: ";
        for (int i = 0;i < relationsEnd - relations;i++)
            oss << std::fixed << std::setw(9) << std::setiosflags(std::ios_base::left) << std::setprecision(6) << timeRelation[i].totalTimeToSeconds();
        return oss.str();
    }
#endif
};
}


