#pragma once

//---------------------------------------------------------------------------
// Wire protocol messages
//---------------------------------------------------------------------------
struct MessageHead {
  /// Message types
  enum Type : uint32_t { Done, DefineSchema, Transaction, ValidationQueries, Flush, Forget };
  /// Total message length, excluding this head
  uint32_t messageLen;
  /// The message type
  Type type;
};
//---------------------------------------------------------------------------
struct DefineSchema {
  /// Number of relations
  uint32_t relationCount;
  /// Column counts per relation, one count per relation. The first column is always the primary key
  uint32_t columnCounts[];
};
//---------------------------------------------------------------------------
struct Transaction {
  /// The transaction id. Monotonic increasing
  uint64_t transactionId;
  /// The operation counts
  uint32_t deleteCount,insertCount;
  /// A sequence of transaction operations. Deletes first, total deleteCount+insertCount operations
  char operations[];
};
//---------------------------------------------------------------------------
struct TransactionOperationDelete {
  /// The affected relation
  uint32_t relationId;
  /// The row count
  uint32_t rowCount;
  /// The deleted values, rowCount primary keyss
  uint64_t keys[];
};
//---------------------------------------------------------------------------
struct TransactionOperationInsert {
  /// The affected relation
  uint32_t relationId;
  /// The row count
  uint32_t rowCount;
  /// The inserted values, rowCount*relation[relationId].columnCount values
  uint64_t values[];
};
//---------------------------------------------------------------------------
struct ValidationQueries {
  /// The validation id. Monotonic increasing
  uint64_t validationId;
  /// The transaction range
  uint64_t from,to;
  /// The query count
  uint32_t queryCount;
  /// The queries
  char queries[];
};
//---------------------------------------------------------------------------
struct Query {
  /// A column description
  struct Column {
    /// Support operations
    enum Op : uint32_t { Equal, NotEqual, Less, LessOrEqual, Greater, GreaterOrEqual };
    /// The column id
    uint32_t column;
    /// The operations
    Op op;
    /// The constant
    uint64_t value;
  };

  /// The relation
  uint32_t relationId;
  /// The number of bound columns
  uint32_t columnCount;
  /// The bindings
  Column columns[];
};
//---------------------------------------------------------------------------
struct Flush {
  /// All validations to this id (including) must be answered
  uint64_t validationId;
};
//---------------------------------------------------------------------------
struct Forget {
  /// Transactions older than that (including) will not be tested for
  uint64_t transactionId;
};

