// vim: set ts=2 sw=2 et:

#pragma once

#include <atomic>
#include <cstdio>
#include <cstdlib>
#include <errno.h>
#include <unistd.h>
#include <sys/syscall.h>
#include "folly/experimental/EventCount.h"

extern char *readBuffer;
extern char *readBufferLimit;
extern std::atomic<char *> readBufferAvailable;
extern std::atomic<int> readBufferFlushEvent;
extern std::atomic<char *> readBufferFlushBase;

extern folly::EventCount waitForBytesEventCount;
extern folly::EventCount flushConsumerCompleteEventCount;

extern const int READ_BUFFER_FLUSH_EVENT_OK;
extern const int READ_BUFFER_FLUSH_EVENT_WAIT_FOR_CONSUMER;
extern const int READ_BUFFER_FLUSH_EVENT_WAIT_FOR_PRODUCER;

void startReaderThread();

#include <iostream>

inline void readBufferWaitForBytes(char *readBufferDemand)
{
  waitForBytesEventCount.await([readBufferDemand](){ return readBufferDemand <= readBufferAvailable.load() || readBufferAvailable.load() == readBufferLimit; });
}

inline void readBufferFlushConsumerComplete(char *flushBase)
{
  readBufferFlushBase.store(flushBase);
  readBufferFlushEvent.store(READ_BUFFER_FLUSH_EVENT_WAIT_FOR_PRODUCER);
  flushConsumerCompleteEventCount.notifyAll();
  flushConsumerCompleteEventCount.await([](){ return readBufferFlushEvent.load() != READ_BUFFER_FLUSH_EVENT_WAIT_FOR_PRODUCER; });
}
