#pragma once

#include <iostream>
#include <cstdint>
#include <cstdlib>
#include <unordered_map>
#include <utility>
#include <malloc.h>
#include "folly/FBVector.h"
#include "folly/experimental/EventCount.h"
#include "memory-pool.h"
#include "xact-history.h"
#include "ptr-hash-map.h"

struct Relation
{
    static constexpr size_t SMALL_ROWS = 128;

    const uint32_t columnCount;
    uint64_t* minValue;
    uint64_t* maxValue;
    uint64_t* historicMinValue;
    uint64_t* historicMaxValue;
    uint64_t firstUnknownXactId;
    Uint64MemoryPool memoryPool;
    PtrHashMap<uint64_t *> activeRows;
    folly::fbvector<XactHistoryRecord> xactHistory;
    uint64_t lastXactHistorySize;
    folly::fbvector<SmallXactHistoryRecord> smallXactHistory;
    int ownerThreadIndex;
    std::atomic<bool> readyForValidation;
    uint64_t approximateCost;
    uint64_t upcomingReservedRows;
    folly::fbvector<std::pair<uint64_t, uint64_t *>> smallPendingRows;

    Relation(uint32_t _columnCount, int _ownerThreadIndex) : columnCount(_columnCount), memoryPool(_columnCount), activeRows(2048), lastXactHistorySize(0), ownerThreadIndex(_ownerThreadIndex), readyForValidation(false), approximateCost(0), upcomingReservedRows(0)
    {
      minValue = new uint64_t[_columnCount];
      maxValue = new uint64_t[_columnCount];
      historicMinValue = new uint64_t[_columnCount];
      historicMaxValue = new uint64_t[_columnCount];
      std::fill_n (minValue, columnCount, std::numeric_limits<uint64_t>::max());
      std::fill_n (maxValue, columnCount, std::numeric_limits<uint64_t>::min());
      std::fill_n (historicMinValue, columnCount, std::numeric_limits<uint64_t>::max());
      std::fill_n (historicMaxValue, columnCount, std::numeric_limits<uint64_t>::min());
    }
};

extern Relation* relations;
extern Relation* relationsEnd;
extern folly::EventCount relationReadyEventCount;
