#pragma once

#include <thread>
#include <iostream>
#include <sstream>
#include "relation.h"
#include "parser.h"
#include "folly/LifoSem.h"
#include "validation-result.h"

#ifdef _PROFILE
#include "profiling.h"
#include <iomanip>
#endif

namespace Transactor
{
extern const int THREAD_COUNT;

struct ThreadContext
{
    ThreadContext(int threadIndex);

    void mainLoop();

    // Parser interface
    void processDone() { }
    void processTransaction(Transaction *t);
    void processValidationQueries(ValidationQueries *v) { }
    void processFlush(Flush *f);
    void processForget(Forget *f);

    void forget(uint64_t forgetTransactionId);

    int threadIndex;
    folly::fbvector<folly::fbvector<std::pair<const ValidationQueries *, const Query *>>> validationQueueItems;
    folly::fbvector<const TransactionOperationDelete *> transactionDeletes;
    folly::fbvector<const TransactionOperationInsert *> transactionInserts;
    folly::fbvector<ValidationResult> validationResults;
    bool shouldForget;
    uint64_t forgetTransactionId;
    folly::LifoSem flushCompleteSemaphore;

#ifdef _PROFILE
    TimestampStack timeStack;
    ProfilingStats timeTransaction;
    ProfilingStats timeTransactionInsert;
    ProfilingStats timeTransactionDelete;
    ProfilingStats timeTransactionDry;
    ProfilingStats timeBuildSmall;
    ProfilingStats timeForget;
    ProfilingStats *timeRelation;

    std::string profilingStatsToString()
    {
        std::ostringstream oss;
        oss << "Transaction: " << timeTransaction.toString() << "\n > Insert: " << timeTransactionInsert.toString() << "\n > Delete: " << timeTransactionDelete.toString() << "\nBuildSmall: " << timeBuildSmall.toString() << "\nForget: " << timeForget.toString() << "\nRelations: ";
        for (int i = 0;i < relationsEnd - relations;i++)
            oss << std::fixed << std::setw(9) << std::setiosflags(std::ios_base::left) << std::setprecision(6) << timeRelation[i].totalTimeToSeconds();
        return oss.str();
    }
#endif
};
}


